/*
 * @(#)BeaconClient.java     0.5  12/08/99
 *
 */

/**
 * BeaconClinet is a class to perform query to BeaconServer
 * It is used by BeaconViewer
 *
 * @version 0.5 12/09/99
 * @author Kai Chen, NLANR
 */

import java.lang.*;
import java.util.*;
import java.net.*;

public class BeaconClient
{
    // final static int maxMonitorPacketSize = 10240;
    final static int maxMonitorPacketSize = 20480;

    public boolean verboseMode;
    public String mcGroup;
    public int mcPort;
    public String beaconServer;
    public int    beaconMonitorPort;
    public String sessionName;

    // BeaconClient can only monitor a session
    // so, no need to have a sessionTable
    public Hashtable reporterTable;

    private DatagramSocket socket = null;
    private byte [] buf = null;   // buffer for reply packet
    private byte [] query = null; // buffer for query packet

    private DatagramPacket dpQuerry = null;
    private InetAddress serverAddr = null;


    BeaconClient (String mGroup, int mPort, String server, int serverPort, boolean verbose) 
	throws BeaconException
    {
	// init the data members

	beaconServer = server;
	beaconMonitorPort = serverPort;
	mcGroup = mGroup;
	mcPort = mPort;
	verboseMode = verbose;

	sessionName =  new String(mcGroup + ":" + mcPort); // construct the session name

       	reporterTable = new Hashtable();

	// build beacon server's address
        try {
            serverAddr = InetAddress.getByName(beaconServer);
        } catch (UnknownHostException ex) {
	    throw new BeaconException("beacon server " + beaconServer + " unknown");
        }

	// create a Datagram socket
	try {
            socket = new DatagramSocket();
	    socket.setSoTimeout(5000); // set SO_TIMEOUT (5 seconds)
	    socket.setReceiveBufferSize(maxMonitorPacketSize); // set SO_RCVBUF
        } catch (SocketException ex) {
	    throw new BeaconException("create socket failed");
        }

        buf = new byte[maxMonitorPacketSize];
	query = new byte[128]; // short just hold the session name

	buildQuerryPacket();
    }

    /*
     * testRun: for testing this class
     */
    /*
    public void testRun()
    {
	try {
	while (true) {
	    System.out.print("> (q or g)");
	    int cmd = System.in.read();
	    if (cmd == 'g') {
		while (System.in.read() != '\n'); // skip the rest of the command line
		     
		// querry - reply
		DatagramPacket dp = new DatagramPacket(buf, buf.length);
		try {
		    socket.send(dpQuerry);
		    System.out.println("beaconClient: querry sent");
		    socket.receive(dp);
		    System.out.println("beaconClient: reply received");
		} catch (java.io.IOException ex) {
		    System.out.println("IOException: " + ex.getMessage());
		    System.out.println("beaconClient: warning receive packet error");
		}
		if (!updateBeaconClientTable(dp)) {
		    System.out.println("beaconClient Error: exit");
		    System.exit(1);
		}
	    }
	    else if (cmd == 'q') { // quit
		return;
	    }
	    else { // unknown
		while (System.in.read() != '\n'); // skip the rest of the command line
	    }
	}
	}
	catch (java.io.IOException ex) {
	    System.out.println("IOException: " + ex.getMessage());
	}

	return; 
    }     
    */

    /*
     * send a query to BeaconServer, get a reply (hopefully), 
     * and update its own data structure
     */

    public void doIt() throws BeaconException
    {
	// querry - reply
	DatagramPacket dp = new DatagramPacket(buf, buf.length);
	try {
	    socket.send(dpQuerry);
	    if (verboseMode) System.out.println("beaconClient: query sent");
	    socket.receive(dp); // timeout 5 seconds
	    if (verboseMode) System.out.println("beaconClient: reply received");
	} catch (java.io.InterruptedIOException ex) {
	    if (verboseMode) System.out.println("InterruptedIOException: " + ex.getMessage());
	    throw new BeaconException("Server no response for 5 seconds.");
	} catch (java.io.IOException ex) {
	    if (verboseMode) System.out.println("IOException: " + ex.getMessage());
	    throw new BeaconException("Packet loss or server down.");
	}
	if (!updateBeaconClientTable(dp)) {
	    // System.out.println("beaconClient Error: exit now");
	    // System.exit(1);
	    throw new BeaconException("Server reply error.");
	}

	return;
    }


    /*
     * clean up 
     */
    protected void finalize()
    {
	socket.close();
    }

    /*
     * build a querry packet since it is re-used each time
     * use the data member "query" as the byte buffer
     */

    public void buildQuerryPacket()
    {
	int len = 0;
	byte [] sbuf = sessionName.getBytes();
	query[len] = (byte)(sbuf.length);
	len += 1;
	for (int i = 0; i < sbuf.length; i++)
	    query[len + i] = sbuf[i];
	len += sbuf.length;

	dpQuerry = new DatagramPacket(query, len, serverAddr, beaconMonitorPort);

	return;
    }



    /*
     * parse the cotents of a reply packet from BeaconServer
     * and update the reporterTable accordingly
     * return: true - updated ok; false - wrong session name
     */

    public boolean updateBeaconClientTable(DatagramPacket dp)
    {
	// parse the reply packet

        byte [] pbuf = dp.getData();

        int pointer = 0;

        // get session name
        byte len = pbuf[pointer];
        String recvSessionName = new String(pbuf, pointer+1, len);
        pointer += len + 1;

	// check the session name
	if (recvSessionName.equals(sessionName)) {
	    if (verboseMode) System.out.println("clientRecvSessionName ok: " + recvSessionName);
	}
	else {
	    if (verboseMode) System.out.println("clientRecvSessionName error: " + recvSessionName);
	    return false;
	}

	int reporterNum = 0; // total of current reporters

	while (pbuf[pointer] != '#') {

	    // get reporter name
	    len = pbuf[pointer];
	    String recvReporterName = new String(pbuf, pointer+1, len);
	    pointer += len + 1;

	    // check the reporter name
	    if (verboseMode) System.out.println("recvReporterName: " + recvReporterName);
	    Hashtable st = (Hashtable) reporterTable.get(recvReporterName); // get sender table
	    if (st == null) { // new reporter (new member joined the session)
		reporterTable.put(recvReporterName, new Hashtable());
		st = (Hashtable) reporterTable.get(recvReporterName);
	    }

	    reporterNum ++; // total of current reporters

	    while (pbuf[pointer] != '*') {

		// get sender name, and its statistics
		len = pbuf[pointer];
		String recvSender = new String(pbuf,pointer+1, len);
		pointer += len + 1;
		if (verboseMode) System.out.println("recvSender: " + recvSender);

		// get lastReceiveTime (long)
		len = pbuf[pointer];
		String tempStr = new String(pbuf, pointer+1, len);
		long recvLastReceiveTime = Long.parseLong(tempStr);
		pointer += len + 1;
		if (verboseMode) System.out.println("recvLastReceiveTime: " + recvLastReceiveTime);

		// get aveDelay (float)
		len = pbuf[pointer];
		tempStr = new String(pbuf, pointer+1, len);
		float recvAveDelay = Float.parseFloat(tempStr);
		pointer += len + 1;
		if (verboseMode) System.out.println("recvAveDelay: " + recvAveDelay);

		// get aveJitter (double)
		len = pbuf[pointer];
		tempStr = new String(pbuf, pointer+1, len);
		double recvAveJitter = Double.parseDouble(tempStr);
		pointer += len + 1;
		if (verboseMode) System.out.println("recvAveJitter: " + recvAveJitter);

		// get aveLossRate (float)
		len = pbuf[pointer];
		tempStr = new String(pbuf, pointer+1, len);
		float recvAveLossRate = Float.parseFloat(tempStr);
		pointer += len + 1;
		if (verboseMode) System.out.println("recvAveLossRate: " + recvAveLossRate);

		// get aveOutOfOrderRate (float)
		len = pbuf[pointer];
		tempStr = new String(pbuf, pointer+1, len);
		float recvAveOutOfOrderRate = Float.parseFloat(tempStr);
		pointer += len + 1;
		if (verboseMode) System.out.println("recvAveOutOfOrderRate: " + recvAveOutOfOrderRate);
		
		// check the sender name
		SenderStat sender = (SenderStat) st.get(recvSender);
		if (sender == null) { // new sender (never reported before)
		    st.put(recvSender, new SenderStat(recvSender));
		    sender = (SenderStat) st.get(recvSender);
		    if (verboseMode) 
			System.out.println("beaconClient: new seen " + recvSender + " from " + recvReporterName);
		}
		
		// update the sender's stat
		sender.updateWith(recvLastReceiveTime,
				  recvAveDelay,
				  recvAveJitter,
				  recvAveLossRate,
				  recvAveOutOfOrderRate);

		if (verboseMode) System.out.println("beaconClient: updated sender " + recvSender);
		
	    } // end of while for senders

	    pointer ++; // skip the '*' flag
	}

	// clean up the reporter table if there is stale reporters
	if (reporterNum != reporterTable.size())
	    cleanReporterTable(dp, reporterNum); 
	
	return true;
    }

    /*
     * clean up the reporter table
     * called when there is staled reporters
     */

    protected void cleanReporterTable(DatagramPacket dp, int reporterNum)
    {
	String [] reporterNameList = new String[reporterNum];

        byte [] pbuf = dp.getData();

        int pointer = 0;

        // skip session name
        byte len = pbuf[pointer];
        pointer += len + 1;

	int index = 0;
        while (pbuf[pointer] != '#') {

            // get reporter name
            len = pbuf[pointer];
            String recvReporterName = new String(pbuf, pointer+1, len);
            pointer += len + 1;

	    reporterNameList[index] = new String(recvReporterName); // keep in array
	    index++;

            while (pbuf[pointer] != '*') {pointer++;} // skip senders
	    pointer ++; // skip '*'
	}

	// clean up
	Enumeration keyList = reporterTable.keys();
	while (keyList.hasMoreElements()) {
	    String aKey = (String) keyList.nextElement();

	    boolean found = false;
	    for (int i = 0; i < reporterNum; i++) 
		if (aKey.equals(reporterNameList[i])) {
		    found = true; 
		    break;
		}
	    
	    if (!found) {
		reporterTable.remove(aKey);
		if (verboseMode) System.out.println("BeaconClient: removed reporter " + aKey);
	    }
	}

	return;
    }

} // end of class











































