/*
 * @(#)BeaconViewer.java     0.51  01/21/2000
 *
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Date;
import java.text.DateFormat;

/**
 * BeaconViewer is a GUI interface to monitor a multicast session via a BeaconServer.
 * There are three components --
 *    Beacon: an active probing program running on a set of machines
 *    BeaconServer: a central server to collect status information from Beacons
 *    BeaconViewer: a GUI program to query the BeaconServer
 *
 * For more information, go to http://dast.nlanr.net
 *
 * @version 0.51 01/21/2000
 * @author Kai Chen, NLANR
 */
 
public class BeaconViewer extends JFrame
				  implements ActionListener
{
    // too frequent query could lead to no time for garbage collection
    // static final int defaultQueryInterval = 10;

    static final int defaultQueryInterval = 60;
    static final String defaultDisplayOption = "loss";
    static final boolean defaultVerboseMode = false;

    int queryInterval;   // query interval to the beacon server
    boolean verboseMode; // verbose mode flag
    String mGroup;
    int mPort;
    String beaconServer;
    int serverPort;

    String displayOption;  // "loss", "delay", "jitter", "out-of-order"
    JPanel beaconPane;     // for displaying beacon info
    boolean windowSized;   // resize the whole apps window when it first starts
    FrequencyDialog frequencyDialog = null; // for change query frequency
    BeaconClient beaconClient = null; // the object to do real work

    // for displaying status info
    JSplitPane statusPane;
    JSplitPane innerPane; 
    JLabel connectionStatus;
    JLabel timeStatus;
    JLabel targetStatus;

    // for image icons
    ImageIcon iconNormal;
    ImageIcon iconYellow;
    ImageIcon iconRed;
    ImageIcon iconInformation;
    ImageIcon iconBlocked;
    

    public BeaconViewer(String [] args) 
    {

	// init the data members
	queryInterval = defaultQueryInterval;
	displayOption = defaultDisplayOption;
	verboseMode = defaultVerboseMode;

	parseCommandLine(args); // must init other arguments in there

	// add window listener to this window

	addWindowListener(new WindowAdapter() {
	    public void windowClosing(WindowEvent e) {
		System.exit(0);
	    }
	});

	// add menu to the window
	createMenu();

	// create the toolbar
	JToolBar toolBar = new JToolBar();
	addButtons(toolBar);

	// create the beacon viewing area
	beaconPane = new JPanel();
	beaconPane.setLayout(new GridLayout(1,1));
	beaconPane.add(new JButton("Please wait ..."));
	windowSized = false;

	// create the status bar area
	createStatus();

	// layout the content pane
	JPanel contentPane = new JPanel();
	contentPane.setLayout(new BorderLayout());
	contentPane.setPreferredSize(new Dimension(400, 100));
	contentPane.add(toolBar, BorderLayout.NORTH);
	contentPane.add(beaconPane, BorderLayout.CENTER);
	contentPane.add(statusPane, BorderLayout.SOUTH);

	setContentPane(contentPane);

	// init the BeaconClient object
	try {
	    beaconClient = new BeaconClient(mGroup, mPort, beaconServer, serverPort, verboseMode);
	} catch (BeaconException ex) {
	    System.out.println("BeaconViewer: BeaconException - " + ex.getMessage());
	    System.exit(1);
	}

	// init the change query frequency dialog	
	frequencyDialog = new FrequencyDialog(this, queryInterval);
	frequencyDialog.pack();

	// init image icons
	iconNormal = new ImageIcon("images/yes.gif");
	iconYellow = new ImageIcon("images/yellow_warning.gif");
	iconRed = new ImageIcon("images/red_warning.gif");
	iconInformation = new ImageIcon("images/i_a.gif");
	iconBlocked = new ImageIcon("images/no_enter.gif");
    }


    /*
     * display the beacon info on the beaonPane
     * return the current updated time (to display on status bar)
     */

    public String updateBeaconPane()
    {

	// get info from reporterTable
	// and display info to the beaconPane

	Hashtable reporterTable = beaconClient.reporterTable;

	// construct th Insets to set the margin
	Insets margin = new Insets(2, 5, 2, 5);

	// build a string array to put on the buttons
	int numReporters = reporterTable.size();
	String [][] tags = new String[numReporters + 1][numReporters + 1]; // same number as senders
	JButton [][] buttons = new JButton[numReporters + 1][numReporters + 1];

	Enumeration reporters = reporterTable.keys(); // all keys for reporters
	String [] labels = new String [numReporters + 1];
	int pos = 1;
	while (reporters.hasMoreElements()) {
	    tags[pos][0] = (String) reporters.nextElement();
	    tags[0][pos] = tags[pos][0];

	    // figure out the text to put on the button
	    int index = tags[pos][0].indexOf('@');
	    String name = tags[pos][0].substring(0, index+1);
	    String host = tags[pos][0].substring(index+1);
	    /* this version puts too much info at the buttons
	    String htmltext = new String("<html><center><i><font color=blue>" + 
					  name + "<br>" + host + 
					  "</font></i></center></html>");

	    buttons[pos][0] = new JButton(htmltext);
	    buttons[0][pos] = new JButton(htmltext);
	    */

	    String domain = host.substring(0, host.lastIndexOf('.'));
	    domain = domain.substring(domain.lastIndexOf('.') + 1);
	    domain = domain.toUpperCase();

	    labels[pos] = domain;

	    pos++;
	}

	// sort the labels[] array
	for (int i = 1; i < numReporters + 1; i++) 
	    for (int j = i; j < numReporters + 1; j++)
		if (labels[i].compareTo(labels[j]) > 0) {
		    String temp = labels[i];
		    labels[i] = labels[j];
		    labels[j] = temp;

		    temp = tags[i][0];
		    tags[i][0] = tags[j][0];
		    tags[j][0] = temp;

		    temp = tags[0][i];
		    tags[0][i] = tags[0][j];
		    tags[0][j] = temp;
		}

	// construct the buttons
	for (int i = 1; i < numReporters + 1; i++) {
	    (buttons[i][0] = new JButton(labels[i])).setMargin(margin);
	    (buttons[0][i] = new JButton(labels[i])).setMargin(margin);
	    buttons[i][0].setBackground(Color.blue);
	    buttons[0][i].setBackground(Color.blue); 
	    buttons[i][0].setToolTipText(tags[i][0]);
	    buttons[0][i].setToolTipText(tags[0][i]);
	}

	if (displayOption.equals("loss")) {
	    // tags[0][0] = new String("Loss Rate (%)");
	    // String htmltext = new String("<html><center>Loss Rate<br>(%)</center></html>");
	    // buttons[0][0] = new JButton(htmltext);
	    (buttons[0][0] = new JButton("L(%)")).setMargin(margin);
	}
	else if (displayOption.equals("delay")) {
            // tags[0][0] = new String("One-Way Delay (ms)");
	    // String htmltext = new String("<html><center>One-Way Delay<br>(ms)</center></html>");
	    // buttons[0][0] = new JButton(htmltext);
	    (buttons[0][0] = new JButton("D(ms)")).setMargin(margin);
	}
        else if (displayOption.equals("jitter")) {
	    // tags[0][0] = new String("Delay Jitter (ms)");
	    // String htmltext = new String("<html><center>Delay Jitter<br>(ms)</center></html>");
	    // buttons[0][0] = new JButton(htmltext);
	    (buttons[0][0] = new JButton("J(ms)")).setMargin(margin);
	}
        else {
	    // tags[0][0] = new String("Out-Of-Order (%)");
	    // String htmltext = new String("<html><center>Out-Of-Order<br>(%)</center></html>");
	    // buttons[0][0] = new JButton(htmltext);
	    (buttons[0][0] = new JButton("O(%)")).setMargin(margin);
	}
	buttons[0][0].setBackground(Color.cyan);

	// get current time, for testing age of the info
	Date now = new Date();
	long currentTime = now.getTime();

	for (int i = 1; i < numReporters + 1; i++) {
	    Hashtable st = (Hashtable) reporterTable.get(tags[i][0]);

	    for (int j = 1; j < numReporters + 1; j++) {
		SenderStat ss = (SenderStat) st.get(tags[0][j]);

		if (ss == null) {
		    // tags[i][j] = new String("N/A") + "  ";
		    tags[i][j] = new String("N/A");
		    (buttons[i][j] = new JButton(tags[i][j])).setMargin(margin);
		    buttons[i][j].setBackground(Color.darkGray);
		    // buttons[i][j].setHorizontalTextPosition(AbstractButton.LEFT);
		    buttons[i][j].setToolTipText("Not reachable from " + tags[0][j] +
						 " to " + tags[i][0]);
		    continue;
		}

		// I don't assume that the machine running BeaconView
		// is clock-synchronized with the machine running Beacon
		// so that I took out the following codes
		
		/*
		long age = currentTime - ss.lastReceiveTime; // milliseconds

 		if (age > 30000) { // older than 30 seconds
		    int age_sec = (int) age / 1000;
		    tags[i][j] = "Stalled (" + (new Integer(age_sec)).toString() + " s)"; 
		}
		else 
		*/
		
		if (displayOption.equals("loss")) {
		    // convert to percentage (times 100)
		    int temp = (int) (ss.aveLossRate * 100); // just need the integer part
		    tags[i][j] = (new Integer(temp)).toString();

		    /* this is the display with icons
		    if (ss.aveLossRate < 0.1) 
			buttons[i][j] = new JButton(tags[i][j], iconNormal);
		    else if (ss.aveLossRate < 0.3)
			buttons[i][j] = new JButton(tags[i][j], iconYellow);
		    else
			buttons[i][j] = new JButton(tags[i][j], iconRed);
		    */

		    // FIX: a version with just text and colored background
		    if (ss.aveLossRate < 0.1) {
			buttons[i][j] = new JButton(tags[i][j]);
			buttons[i][j].setBackground(Color.green);
		    }
		    else if (ss.aveLossRate < 0.3) {
			buttons[i][j] = new JButton(tags[i][j]);
			buttons[i][j].setBackground(Color.yellow);
		    }
		    else {
			buttons[i][j] = new JButton(tags[i][j]);
			buttons[i][j].setBackground(Color.red);
		    }
		    // buttons[i][j].setHorizontalTextPosition(AbstractButton.LEFT);
		    buttons[i][j].setToolTipText("Loss Rate = " + tags[i][j] + "%" + 
						 " from " + tags[0][j] +
						 " to " + tags[i][0]);


		}
		else if (displayOption.equals("delay")) {
		    // just need the integer part
		    int temp = (int) ss.aveDelay;
		    // tags[i][j] = (new Float(ss.aveDelay)).toString() + "  " ;
		    tags[i][j] = (new Integer(temp)).toString();
		    // buttons[i][j] = new JButton(tags[i][j], iconInformation);
		    (buttons[i][j] = new JButton(tags[i][j])).setBackground(Color.gray);
		    // buttons[i][j].setHorizontalTextPosition(AbstractButton.LEFT);
		    buttons[i][j].setToolTipText("Delay = " + tags[i][j] + "ms" + 
						 " from " + tags[0][j] +
						 " to " + tags[i][0]);
		}
		else if (displayOption.equals("jitter")) {
		    int temp = (int) (ss.aveJitter * 10);
		    float temp2 = (float) temp / 10;
		    tags[i][j] = (new Float(temp2)).toString();
		    // buttons[i][j] = new JButton(tags[i][j], iconInformation);
		    (buttons[i][j] = new JButton(tags[i][j])).setBackground(Color.gray);
		    // buttons[i][j].setHorizontalTextPosition(AbstractButton.LEFT);
		    buttons[i][j].setToolTipText("Jitter = " + tags[i][j] + "ms" + 
						 " from " + tags[0][j] +
						 " to " + tags[i][0]);
		}
		else { // out-of-order
		    // convert to percentage (times 100)
		    float temp = ((int)(ss.aveOutOfOrderRate * 1000)) / (float) 10.0;
		    tags[i][j] = (new Float(temp)).toString();
		    
		    /* this is the version with icons
		    if (ss.aveOutOfOrderRate < 0.01) 
			buttons[i][j] = new JButton(tags[i][j], iconNormal);
		    else if (ss.aveLossRate < 0.05)
			buttons[i][j] = new JButton(tags[i][j], iconYellow);
		    else
			buttons[i][j] = new JButton(tags[i][j], iconRed);
		    */

		    // FIX: a version without icons
		    if (ss.aveOutOfOrderRate < 0.01) {
			buttons[i][j] = new JButton(tags[i][j]);
			buttons[i][j].setBackground(Color.green);
		    }
		    else if (ss.aveOutOfOrderRate < 0.05) {
			buttons[i][j] = new JButton(tags[i][j]);
			buttons[i][j].setBackground(Color.yellow);
		    }
		    else {
			buttons[i][j] = new JButton(tags[i][j]);
			buttons[i][j].setBackground(Color.red);
		    }
	    
		    // buttons[i][j].setHorizontalTextPosition(AbstractButton.LEFT);
		    buttons[i][j].setToolTipText("Out-of-order Rate = " + tags[i][j] + "%" + 
						 " from " + tags[0][j] +
						 " to " + tags[i][0]);
		}
		// make the margin smaller than default
		buttons[i][j].setMargin(margin);
	    }
	}

	beaconPane.removeAll(); // remove all the components
	beaconPane.setLayout(new GridLayout(numReporters + 1, numReporters + 1));
	for (int i = 0; i < numReporters + 1; i++) 
	    for (int j = 0; j < numReporters + 1; j++) {
	    beaconPane.add(buttons[i][j]);
	}
	beaconPane.validate(); // invalidate so that the 

	// resize the window according to the number of reporters
	// this is done once when starts up
	// it doesn't get changed even if the number of reporters changed during running
	if (! windowSized) {
	    int width = 700;
	    int height = 300;
	    if (numReporters > 2) {
		width += (numReporters - 2) * 20;
		height += (numReporters - 2) * 10;
	    }
	    this.setSize(width, height);
	    windowSized = true;
	}

	// FIX: make the buttons[][] pointers to null to facilitate garbage collection
	for (int i = 0; i < numReporters + 1; i++)
	    for (int j = 0; j < numReporters + 1; j++)
		buttons[i][j] = null;

	return DateFormat.getTimeInstance(DateFormat.FULL).format(now);
    }

    /*
     * create the status pane at the bottom of screen
     */
    protected void createStatus()
    {
	connectionStatus = new JLabel("For display status of connection.");
	timeStatus = new JLabel("next: 00:00");
	targetStatus = new JLabel(mGroup + ":" + mPort + " via " + beaconServer);
	
	innerPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
					      timeStatus, targetStatus);
	innerPane.setDividerLocation(150);

	statusPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
					      connectionStatus, innerPane);
	statusPane.setDividerLocation(450);

	return;
    }

    /* 
     * set status at the connection area
     */
    protected void setConnectionStatus(String s)
    {
	connectionStatus.setText(s);
	return;
    }

    /* 
     * set status at the next query time area
     */
    protected void setTimeStatus(String s)
    {
	timeStatus.setText(s);
	return;
    }

    /* 
     * set status at the monitoring target area
     */
    protected void setTargetStatus(String s)
    {
	targetStatus.setText(s);
	return;
    }

    /*
     * add buttons on the tool bar area
     */
    protected void addButtons(JToolBar toolBar) {
        JButton button = null;

        //first button
        button = new JButton("Loss", new ImageIcon("images/button_loss.gif"));
        button.setVerticalTextPosition(AbstractButton.BOTTOM);
        button.setHorizontalTextPosition(AbstractButton.CENTER);
	button.setToolTipText("view loss rate");
        button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
		if (verboseMode) System.out.println("toolbar action: view loss rate");
		if (!displayOption.equals("loss")) {
		    displayOption = new String ("loss");
		    updateBeaconPane();
		}		    
            }
        });
        toolBar.add(button);

        //second button
        button = new JButton("delay", new ImageIcon("images/button_delay.gif"));
        button.setVerticalTextPosition(AbstractButton.BOTTOM);
        button.setHorizontalTextPosition(AbstractButton.CENTER);
        button.setToolTipText("view delay");
        button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
		if (verboseMode) System.out.println("toolbar action: view delay");
		if (!displayOption.equals("delay")) {
		    displayOption = new String ("delay");
		    updateBeaconPane();
		}		    
            }
        });
        toolBar.add(button);

        //third button
        button = new JButton("jitter", new ImageIcon("images/button_jitter.gif"));
        button.setVerticalTextPosition(AbstractButton.BOTTOM);
        button.setHorizontalTextPosition(AbstractButton.CENTER);
        button.setToolTipText("view delay jitter");
        button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
		if (verboseMode) System.out.println("toolbar action: view delay jitter");
		if (!displayOption.equals("jitter")) {
		    displayOption = new String ("jitter");
		    updateBeaconPane();
		}		    
            }
        });
        toolBar.add(button);

        //fourth button
        button = new JButton("order", new ImageIcon("images/button_ooo.gif"));
        button.setVerticalTextPosition(AbstractButton.BOTTOM);
        button.setHorizontalTextPosition(AbstractButton.CENTER);
        button.setToolTipText("view out-of-order rate");
        button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
		if (verboseMode) System.out.println("toolbar action: view out-of-order rate");
		if (!displayOption.equals("out-of-order")) {
		    displayOption = new String ("out-of-order");
		    updateBeaconPane();
		}		    
            }
        });
        toolBar.add(button);

	return;
    }

    /*
     * create menu for the main frame
     */
    public void createMenu()
    {
	JMenuBar menuBar;
	JMenu menu;
	JMenuItem menuItem;

	// create the menu bar
	menuBar = new JMenuBar();
	setJMenuBar(menuBar);

	// build the first menu
	menu = new JMenu("File");
	menu.setMnemonic(KeyEvent.VK_F);
	menuBar.add(menu);

	//a group of JMenuItems
	menuItem = new JMenuItem("Exit");
        menuItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_E, ActionEvent.ALT_MASK));
        menuItem.addActionListener(this);
        menu.add(menuItem);

	// build the second menu
	menu = new JMenu("Options");
	menu.setMnemonic(KeyEvent.VK_O);
	menuBar.add(menu);

	//a group of JMenuItems
	menuItem = new JMenuItem("Query Interval");
        menuItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_Q, ActionEvent.ALT_MASK));
        menuItem.addActionListener(this);
        menu.add(menuItem);

	menuItem = new JMenuItem("Monitoring Target");
        menuItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_M, ActionEvent.ALT_MASK));
        menuItem.addActionListener(this);
        menu.add(menuItem);
	
	// build the third menu
	menu = new JMenu("Help");
        menu.setMnemonic(KeyEvent.VK_H);
        menuBar.add(menu);

	//a group of JMenuItems
	menuItem = new JMenuItem("About");
        menuItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_A, ActionEvent.ALT_MASK));
        menuItem.addActionListener(this);
        menu.add(menuItem);

    }	

    /*
     * action listener for menu item events
     */
    public void actionPerformed(ActionEvent e) {
        JMenuItem source = (JMenuItem)(e.getSource());
        String s = source.getText();

	if (verboseMode) System.out.println("Menu Event: " + s);

	if (s.equals("Exit")) {
	    System.exit(0);
	}
	else if (s.equals("Query Interval")) {
	    frequencyDialog.setLocationRelativeTo(this); 
	    frequencyDialog.setVisible(true);

	    int result = frequencyDialog.getValidatedFrequency();
	    if (result == 0) {
		if (verboseMode) System.out.println("Change Frequency: Cancelled");
	    } else {
		queryInterval = result;
		if (verboseMode) System.out.println("Change Frequency: new value = " + queryInterval);
	    }
	}
	else if (s.equals("Monitoring Target")) {
	    JOptionPane.showMessageDialog(this,
					  "Sorry, changing target is not yet implemented.\n\n" +
					  "You have to specify the session and server name\n" +
					  "at the command line. Use -help for usage.\n\n" +
					  "You may request this feature to dast@nlanr.net.\n",
					  "Change Monitoring Target",
					  JOptionPane.INFORMATION_MESSAGE);
	}
	else if (s.equals("About")) {
	    JOptionPane.showMessageDialog(this,
					  "Multicast BeaconViewer v0.51\n" +
					  "Supported by\n" +
					  "National Laboratory for Applied Network Research\n\n" +
					  "Comments: dast@nlanr.net\n" +
					  "More info: http://dast.nlanr.net",
					  "About",
					  JOptionPane.INFORMATION_MESSAGE);
	}

	return;
    }

    /*
     * let the BeconClient do the query-reply-update
     */
    public void refreshClient()
    {
	setConnectionStatus("Talking to server ...");
	try {
	    beaconClient.doIt(); // do the query-reply, possibly exceptions
	    setConnectionStatus("Server reply received.");
	    String updateTime = updateBeaconPane(); // display it on the screen
	    setConnectionStatus("Updated " + updateTime);
	} catch (BeaconException ex) {
	    setConnectionStatus(ex.getMessage());
	}

	return;
    }

    /*
     * sleep for an interval time
     * updates the countdown clock in status bar
     */
    public void waitNext()
    {
	int counter = queryInterval;

	while (counter > 0) {
	    int min = counter / 60;
	    int sec = counter - min * 60;
	    int min_1 = min / 10;
	    int min_2 = min - min_1 * 10;
	    int sec_1 = sec / 10;
	    int sec_2 = sec - sec_1 * 10;

	    setTimeStatus("next: " + min_1 + min_2 +":" + sec_1 + sec_2);

	    try {
		Thread.sleep(1000); // one second 
	    } catch (InterruptedException ex) {
		ex.printStackTrace();
	    }
	    counter --;
	}
	return;
    }

    /*
     * parse command line arguments
     */
    private void parseCommandLine(String [] args)
    {
	int total = 0; // total required arguments parsed

        int i = 0;
        while (i < args.length) {
            if (args[i].equals("-group")) {
                if (i + 1 < args.length) {
                    i++;
                    mGroup = args[i];
                }
                else {
                    System.out.println("BeaconViewer: multicast group not specified");
                    usageQuit(); // parse error
                }
		total ++;
            }
            else if (args[i].equals("-port")) {
                if (i + 1 < args.length) {
                    i++;
                    mPort = Integer.parseInt(args[i]);
                }
                else {
                    System.out.println("BeaconViewer: multicast port not specified");
                    usageQuit(); // parse error
                }
		total ++;
            }
            else if (args[i].equals("-server")) {
                if (i + 1 < args.length) {
                    i++;
		    beaconServer = args[i];
                }
                else {
                    System.out.println("BeaconViewer: beacon server not specified");
                    usageQuit(); // parse error
                }
		total ++;
            }
            else if (args[i].equals("-serverport")) {
                if (i + 1 < args.length) {
                    i++;
                    serverPort = Integer.parseInt(args[i]);
                }
                else {
                    System.out.println("BeaconViewer: beacon server port not specified");
                    usageQuit(); // parse error
                }
		total ++;
            }
            else if (args[i].equals("-interval")) {
                if (i + 1 < args.length) {
                    i++;
                    queryInterval = Integer.parseInt(args[i]);
                }
                else {
                    System.out.println("BeaconViewer: query interval not specified");
                    usageQuit(); // parse error
                }
            }
            else if (args[i].equals("-verbose")) {
		verboseMode = true;
            }
            else if (args[i].equals("-help")) {
		usageQuit();
            }
            else { //unknown
                System.out.println("BeaconViewer: invalid argument " + args[i]);
                usageQuit();
            }

            i++;

        }

	if (total != 4) {
	    System.out.println("BeaconViewer: must supply all required arguments");
	    usageQuit(); // total of 4 arguments required
	}

        return;
    }

    /*
     * print usage message then quit
     */
    private void usageQuit()
    {
        System.out.println(
	    "Usage: java BeaconViewer -group g -port p -server s -serverport e [-interval i] [-verbose]\n" +
            "\t-group      g  multicast group\n" +
            "\t-port       p  multicast port\n" +
            "\t-server     s  beacon server name\n" +
	    "\t-serverport e  beacon server port\n" +
	    "\t-interval   i  query interval in seconds, default to " + defaultQueryInterval + "\n" +
	    "\t-verbose       verbose mode, default to not-verbose\n");

        System.exit(1);
    }

    /*
     * the main function
     */
    public static void main(String[] args)
    {
	BeaconViewer viewer = new BeaconViewer(args);

	viewer.setTitle("Beacon Viewer");
	viewer.setSize(700, 300);
	viewer.setVisible(true);

	// loop to perform refreshing
	while (true) {
	    viewer.refreshClient();
	    viewer.waitNext();
	}

    } // end of main

} // end of class

















